@@ -1,5 +1,6 @@
     package Math::GMPf;
     use strict;
+    use warnings;
     use Math::GMPf::Random;
     require Exporter;
     *import = \&Exporter::import;
@@ -75,8 +76,10 @@ fgmp_randinit_default fgmp_randinit_mt fgmp_randinit_lc_2exp fgmp_randinit_lc_2e
 fgmp_randinit_set fgmp_randinit_default_nobless fgmp_randinit_mt_nobless
 fgmp_randinit_lc_2exp_nobless fgmp_randinit_lc_2exp_size_nobless fgmp_randinit_set_nobless
 fgmp_urandomb_ui fgmp_urandomm_ui
+Rmpf_get_NV Rmpf_set_NV
+Rmpf_get_IV Rmpf_set_IV Rmpf_fits_UV_p Rmpf_fits_IV_p
     );
-    our $VERSION = '0.41';
+    our $VERSION = '0.42';
     #$VERSION = eval $VERSION;
 
     DynaLoader::bootstrap Math::GMPf $VERSION;
@@ -108,8 +111,13 @@ fgmp_randinit_default fgmp_randinit_mt fgmp_randinit_lc_2exp fgmp_randinit_lc_2e
 fgmp_randinit_set fgmp_randinit_default_nobless fgmp_randinit_mt_nobless
 fgmp_randinit_lc_2exp_nobless fgmp_randinit_lc_2exp_size_nobless fgmp_randinit_set_nobless
 fgmp_urandomb_ui fgmp_urandomm_ui
+Rmpf_get_NV Rmpf_set_NV
+Rmpf_get_IV Rmpf_set_IV Rmpf_fits_UV_p Rmpf_fits_IV_p
 )]);
 
+$Math::GMPf::NOK_POK = 0; # Set to 1 to allow warnings in new() and overloaded operations when
+                          # a scalar that has set both NOK (NV) and POK (PV) flags is encountered
+
 sub dl_load_flags {0} # Prevent DynaLoader from complaining and croaking
 
 sub new {
@@ -157,6 +165,19 @@ sub new {
       return Rmpf_init_set_str($arg1, 10);
     }
 
+    if($type == _POK_T) {
+      if(@_ > 1) {die "Too many arguments supplied to new() - expected no more than two"}
+      if(_SvNOK($arg1)) {
+        set_nok_pok(nok_pokflag() + 1);
+        if($Math::GMPf::NOK_POK) {
+          warn "Scalar passed to new() is both NV and PV. Using PV (string) value";
+        }
+      }
+      $base = shift if @_;
+      if(($base < 2 && $base > -2) || $base < -62 || $base > 62) {die "Invalid value for base"}
+      return Rmpf_init_set_str($arg1, $base);
+    }
+
     if($type == _NOK_T) {
       if(@_ ) {die "Too many arguments supplied to new() - expected only one"}
       if(Math::GMPf::_has_longdouble()) {
@@ -165,14 +186,6 @@ sub new {
         return $ret;
       }
       return Rmpf_init_set_d($arg1);
-
-    }
-
-    if($type == _POK_T) {
-      if(@_ > 1) {die "Too many arguments supplied to new() - expected no more than two"}
-      $base = shift if @_;
-      if(($base < 2 && $base > -2) || $base < -62 || $base > 62) {die "Invalid value for base"}
-      return Rmpf_init_set_str($arg1, $base);
     }
 
     if($type == _MATH_GMPf_T) {
@@ -300,6 +313,16 @@ sub Rmpf_snprintf {
     return $len;
 }
 
+# _Rmpz_get_IV may have returned a "string" - in which case we want to coerce it
+# to an IV. It may be more efficient to do this in XS space (TODO), but in the
+# meantime I've taken the soft option of having perl perform the coercion:
+
+sub Rmpf_get_IV {
+   my $ret = _Rmpf_get_IV(shift);
+   $ret += 0 unless _SvIOK($ret); # Set the IV flag
+   return $ret;
+}
+
 sub __GNU_MP_VERSION {return ___GNU_MP_VERSION()}
 sub __GNU_MP_VERSION_MINOR {return ___GNU_MP_VERSION_MINOR()}
 sub __GNU_MP_VERSION_PATCHLEVEL {return ___GNU_MP_VERSION_PATCHLEVEL()}
@@ -541,6 +564,9 @@ __END__
    Rmpf_set_ui($rop, $ui);
    Rmpf_set_si($rop, $si);
    Rmpf_set_d($rop, $double);
+   Rmpf_set_NV($rop, $NV); # $NV is $Config{nvtype}
+   Rmpf_set_IV($rop, $IV); # $IV is $Config{ivtype}
+                           # (or $Config{uvtype}).
    Rmpf_set_z($rop, $z); # $z is a Math::GMPz object.
    Rmpf_set_q($rop, $q); # $q is a Math::GMPq object.
     Set the value of $rop from the 2nd arg.
@@ -616,12 +642,22 @@ __END__
    $double = Rmpf_get_d($op);
     Convert $op to a 'double'.
 
+   $NV = Rmpf_get_NV($op); # $NV is $Config{nvtype}
+    Convert $op to an NV.
+
    $si = Rmpf_get_si($op);
    $ui = Rmpf_get_ui($op);
     Convert $op to a `signed long' or `unsigned long',
     truncating any fraction part.  If $op is too big for
     the return type, the result is undefined.
 
+   $IV = Rmpf_get_IV($op);
+    If $op (truncated to an integer value) fits into either an
+    IV or a UV return that IV/UV value of (truncated) $op.
+    Otherwise die with an appropriate error message. To find
+    find out if the truncated value of $op will fit, use the
+    functions 'Rmpf_fits_IV_p' and 'Rmpf_fits_UV_p'.
+
    ($double, $exp) = Rmpf_get_d_2exp($op);
     Find $double and $exp such that $double * (2 ** $exp),
     with 0.5<=abs($double)<1, is a good approximation to $op.
@@ -789,8 +825,25 @@ __END__
    $bool = Rmpf_fits_sint_p($op);
    $bool = Rmpf_fits_ushort_p($op);
    $bool = Rmpf_fits_sshort_p($op);
-    Return non-zero if OP would fit in the respective C data
-    type, when truncated to an integer.
+   $bool = Rmpf_fits_UV_p($op);    # fits into a perl UV
+   $bool = Rmpf_fits_IV_p($op);    # fits into a perl IV
+    Return non-zero if OP would fit in the respective type, when
+    truncated to an integer.
+
+   $iv = Math::GMPf::nok_pokflag(); # not exported
+    Returns the value of the nok_pok flag. This flag is
+    initialized to zero, but incemented by 1 whenever a
+    scalar that is both a float (NOK) and string (POK) is passed
+    to new() or to an overloaded operator. The value of the flag
+    therefore tells us how many times such events occurred . The
+    flag can be reset to 0 by running clear_nok_pok().
+
+   Math::GMPf::set_nok_pok($iv); # not exported
+    Resets the nok_pok flag to the value specified by $iv.
+
+   Math::GMPf::clear_nok_pok(); # not exported
+    Resets the nok_pok flag to 0.(Essentially the same
+    as running set_nok_pok(0).)
 
    #######################
 
@@ -966,14 +1019,14 @@ __END__
        -Duse64bitint, the variable is treated as a signed long long
        int if the IOK flag is set.)
 
-    3. If the variable is a double, then that value is used. The
-       variable is considered to be a double if the NOK flag is set.
-
-    4. If the variable is a string (ie the POK flag is set) then the
+    3. If the variable is a string (ie the POK flag is set) then the
        base 10 value of that string is used. If the POK flag is set,
        but the string is not a valid base 10 number, the subroutine
        croaks with an appropriate error message.
 
+    4. If the variable is a double, then that value is used. The
+       variable is considered to be a double if the NOK flag is set.
+
     5. If the variable is a Math::GMPf object (or, for operators
        specified above, a Math::MPFR object) then the value of that
        object is used.
